*************************************************************************************
*														*
*	Mandelbrot set fractals for EASy68K V1.00				5/10/2008		*
*														*
*	This program allows limited exploration of the Mandelbrot set and lets you	*
*	save the images generated during the exploration.					*
*														*
*	This program is not fast, it can take from a few minutes to a few hours to	*
*	generate a new image depending on the window size and the iteration terminal	*
*	count.												*
*														*
*	There is a limit to how far in you can zoom. This limit is reached when the	*
*	delta between pixels becomes smaller than the smallest number that can be	*
*	represented in a 6.26 fixed point number. When this happens the delta is	*
*	set to this smallest number and no further zoom is possible. To go beyond	*
*	this point would require the use of more bits to represent values.		*
*														*
*	To zoom into the image select a box by clicking on one corner of the box,	*
*	move the mouse to the opposite corner of the box and click again to set		*
*	the box, then press the [z] key to zoom in. Zoom out is the same except		*
*	press the [u] key. Clicking with the right mouse button or hitting the		*
*	[ESC] key will abort the current box.							*
*														*
*	To generate an image, if one did not start after your last action or you	*
*	aborted the last one, press the [g] key.							*
*														*
*	To abort the generation of an image hit the [ESC] key. You do not need to	*
*	wait for a complete image to change the screen size, the iteration terminal	*
*	count or to zoom in or out.									*
*														*
*	To change the iteration terminal count press the [i] key and enter a new	*
*	value. To quit without changing the value enter a value of 0 or no value	*
*	at all. If the iteration terminal count has changed a new image will be		*
*	generated.												*
*														*
*	To save an image press the [s] key once the image is complete. A save image	*
*	file request box will appear.									*
*														*
*	The F2, F3 and F4	keys can be used to select a screen size of 640 x 480,	*
*	800 x 600 and 1024 x 768 respectively.							*
*														*
*														*
*	More 68000 and other projects can be found on my website at ..			*
*														*
*	  http://www.themotionstore.com/leeedavison/index.html				*
*														*
*	.. or ..												*
*														*
*	  http://members.lycos.co.uk/leeedavison/index.html					*
*														*
*	mail : leeedavison@lycos.co.uk								*
*														*
*************************************************************************************
*
* first some constants

* real boundary for image

real_max	EQU $04400000			* = +1.06250 as a 6.26 fixed point number
real_min	EQU $F7400000			* = -2.18750 as a 6.26 fixed point number

* imaginary boundary for image

imag_max	EQU $04E00000			* = +1.21875 as a 6.26 fixed point number
imag_min	EQU $FB200000			* = -1.21875 as a 6.26 fixed point number

* iteration boundary for image

iter_min	EQU 50				* minimum iteration bound
iter_max	EQU 255				* maximum iteration bound


*************************************************************************************
*
* the code

	ORG		$1000				* set origin

start
	MOVEQ		#0,d1				* turn off echo
	MOVEQ		#12,d0			* keyboard echo
	TRAP		#15

	MOVE.w	#$FF00,d1			* clear screen
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	BSR		Initialise			* go setup everything
	BSR		generate			* generate the initial fractal image

loop
	BSR		screen_size			* test and handle widow size changes
	BSR		mouse_buttons		* see if the buttons were clicked
	BSR		mouse_move			* see if the mouse moved

	MOVEQ		#7,d0				* read the key status
	TRAP		#15

	TST.b		d1				* test the result
	BEQ.s		loop				* if no key waiting skip the key tests

	PEA		loop(pc)			* push the key function return address

	MOVEQ		#5,d0				* read a key
	TRAP		#15

	ANDI.b	#$DF,d1			* ensure upper case

	CMPI.b	#'G',d1			* compare with "G"
	BNE.s		not_g				* if not "G" skip generate

	BRA		generate			* generate the fractal

not_g
	CMPI.b	#'R',d1			* compare with "R"
	BNE.s		not_r				* if not "R" skip redraw

	BRA		redraw			* redraw the fractal

not_r
	CMPI.b	#'Z',d1			* compare with "Z"
	BNE.s		not_z				* if not "Z" skip zoom

	BRA		do_zoom			* zoom in

not_z
	CMPI.b	#'U',d1			* compare with "U"
	BNE.s		not_u				* if not "U" skip zoom

	BRA		do_unzoom			* zoom out

not_u
	CMPI.b	#'S',d1			* compare with "S"
	BNE.s		not_s				* if not "S" skip save

	BRA		save_image			* save the image

not_s
	CMPI.b	#'I',d1			* compare with "I"
	BNE.s		not_i				* if not "S" skip save

	BRA		iterations			* go set the iteration count

not_i
	CMPI.b	#$1B,d1			* compare with [ESC]
	BNE.s		not_esc			* if not [ESC] skip reset box

	BSR		rubberband			* erase the rubber band
	MOVEQ		#-1,d0			* unset x,y
	MOVE.l	d0,firstX(a3)		* unset the first co-ordinate pair
	MOVE.l	d0,secondX(a3)		* unset the last co-ordinate pair
not_esc
	RTS						* loop forever


*************************************************************************************
*
* check the [F2], [F3] and [F4] keys. set the screen size to 640 x 480, 800 x 600 or
* 1024 x 768 if the corresponding key has been pressed

screen_size
	MOVE.l	#$71007273,d1		* [F2], [], [F3] and [F4] keys
	MOVEQ		#19,d0			* check for keypress
	TRAP		#15

	MOVE.l	d1,d2				* copy result
	BEQ.s		notscreen			* skip screen size if no F key

	MOVE.l	#$028001E0,d1		* set 640 x 480
	TST.l		d2				* test result
	BMI		setscreen			* if F2 go set window size

	MOVE.l	#$03200258,d1		* set 800 x 600
	TST.w		d2				* test result
	BMI		setscreen			* if F3 go set window size

							* else was F4 so ..
	MOVE.l	#$04000300,d1		* set 1024 x 768
setscreen
	CMP.l		screen_width(a3),d1	* compare with current screen size
	BEQ.s		notscreen			* if already set skip setting it now

	MOVE.l	d1,-(sp)			* save d1
	BSR		rubberband			* erase any rubber band
	MOVEQ		#-1,d0			* unset x,y
	MOVE.l	d0,firstX(a3)		* unset the first co-ordinate pair
	MOVE.l	d0,secondX(a3)		* unset the last co-ordinate pair
	MOVE.l	(sp)+,d1			* restore d1
screenset
	MOVE.l	d1,screen_width(a3)	* save the window size
	MOVEQ 	#33,d0			* get/set window size
	TRAP		#15

	PEA		redraw(pc)			* return via the redraw routine
*##notscreen
*##	RTS


*************************************************************************************
*
* calculate the new delta

do_delta
	MOVE.l	rmax(a3),d0			* get the real (x) value maximum
	SUB.l		rmin(a3),d0			* subtract the real (x) value minimum

	MOVE.l	#$00666666,d1		* 1/640 in 0.32 fixed point
	CMPI.w	#640,xsize(a3)		* compare width with 640
	BEQ.s		find_delta			* if 640 wide go find delta

	MOVE.l	#$0051EB85,d1		* 1/800 in 0.32 fixed point
	CMPI.w	#800,xsize(a3)		* compare width with 800
	BEQ.s		find_delta			* if 800 wide go find delta

	MOVE.l	#$00400000,d1		* 1/1024 in 0.32 fixed point
find_delta
	BSR		multiply			* go find the delta
	BNE.s		save_delta			* if not zero go save it

	MOVEQ		#1,d0				* else set to smallest delta
save_delta
	MOVE.l	d0,delta(a3)		* save the delta
notscreen
	RTS


*************************************************************************************
*
* set the iteration count. if the value entered is between iter_min and iter_max
* inclusive then the new iteration terminal count is set. if a value of zero is
* entered or no value is entered the iteration terminal count is not changed. If a
* value outside the range iter_min to iter_max is entered then the routine loops
* for another value.

iterations
	MOVEQ		#0,d1				* top left
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	LEA		count_msg(pc),a1		* point to the message
	MOVEQ		#14,d0			* display the message
	TRAP		#15

	MOVE.w	#$1E01,d1			* after '255. '
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	MOVEQ		#1,d1				* turn on echo
	MOVEQ		#12,d0			* keyboard echo
	TRAP		#15

	MOVEQ		#4,d0				* get a value in d1.l
	TRAP		#15

	TST.w		d1				* test the returned number
	BEQ.s		no_new_exit			* if zero just exit

	CMPI.w	#iter_min,d1		* compare the entered value with the minimum
	BCS.s		iterations			* if < minimum go get another value

	CMPI.w	#iter_max,d1		* compare the entered value with the maximum
	BHI.s		iterations			* if > maximum go get another value

	CMP.w		iter(a3),d1			* compare the old count with the new count
	BEQ.s		no_new_exit			* if the same just exit

	MOVE.w	d1,iter(a3)			* save the new iteration terminal count
	PEA		generate(pc)		* set return to go generate the fractal
no_new_exit
	MOVEQ		#0,d1				* turn off echo
	MOVEQ		#12,d0			* keyboard echo
	TRAP		#15

	MOVEQ		#0,d1				* top left
	MOVEQ		#11,d0			* position cursor
	TRAP		#15

	LEA		count_blk(pc),a1		* point to the blank message
	MOVEQ		#14,d0			* display the message
	TRAP		#15

	BRA		redraw			* redraw the image		

count_msg
	dc.b	'Set the iteration terminal count.   ',$0D,$0A
	dc.b	'Enter a value from 50 to 255.       ',0

count_blk
	dc.b	'                                    ',$0D,$0A
	dc.b	'                                    ',0

	ds.w	0					* ensure even


*************************************************************************************
*
* zoom the image and, if it was a valid zoom, generate the new image

do_zoom
	MOVE.w	firstX(a3),d1		* test if the first x,y pair is set
	BMI		exit_do_zoom		* if not just exit

	MOVE.w	SecondX(a3),d0		* test if the second x,y pair is set
	BMI		exit_do_zoom		* if not just exit

* first and second x,y pair are set so calculate the new values for the zoom

	CMP.w		d1,d0				* compare first with second
	BCC.s		no_exg_x			* if first < second skip the exchange

	EXG		d1,d0				* else exchange first and second x
no_exg_x
	BSR		n_delta			* get the second x * delta
	ADD.l		rmin(a3),d0			* add the start x
	MOVE.l	d0,rmax(a3)			* save the new maximum real value

	MOVE.w	d1,d0				* copy the first x

	BSR		n_delta			* get the first x * delta
	ADD.l		rmin(a3),d0			* add the start x
	MOVE.l	d0,rmin(a3)			* save the new minimum real value

* done rmax and rmin, now do imax and imin

							* remember, 0 is the bottom of the screen
	MOVE.w	ysize(a3),d1		* get the y size
	SUBQ.w	#1,d1				* - 1
	MOVE.w	d1,d0				* copy it

	SUB.w		firstY(a3),d1		* get and invert the first y value
	SUB.w		SecondY(a3),d0		* get and invert the second y value

	CMP.w		d1,d0				* compare first with second
	BCC.s		no_exg_y			* if first < second skip the exchange

	EXG		d1,d0				* else exchange first and second y
no_exg_y
	BSR		n_delta			* get the second y * delta
	ADD.l		imin(a3),d0			* add the start y
	MOVE.l	d0,imax(a3)			* save the new maximum imaginary value

	MOVE.w	d1,d0				* copy the first y

	BSR		n_delta			* get the first y * delta
	ADD.l		imin(a3),d0			* add the start y
	MOVE.l	d0,imin(a3)			* save the new minimum imaginary value
	BSR		rubberband			* erase the rubber band
	MOVEQ		#-1,d0			* unset x,y
	MOVE.l	d0,firstX(a3)		* unset the first co-ordinate pair
	MOVE.l	d0,secondX(a3)		* unset the last co-ordinate pair
	BSR		do_delta			* go calculate the new deltas
	BRA		generate			* and generate the fractal

exit_do_zoom
	RTS


*************************************************************************************
*
* unzoom the image and, if it was a valid unzoom, generate the new image

do_unzoom
	MOVE.w	firstX(a3),d1		* test if the first x,y pair is set
	BMI		exit_do_unzoom		* if not just exit

	MOVE.w	SecondX(a3),d0		* test if the second x,y pair is set
	BMI		exit_do_unzoom		* if not just exit

* first and second x,y pair are set so calculate the new values for the zoom

	CMP.w		d1,d0				* compare first with second
	BCC.s		no_x_exg			* if first < second skip the exchange

	EXG		d1,d0				* else exchange first and second x
no_x_exg
	BSR		n_delta			* get the second x * delta
	ADD.l		rmax(a3),d0			* add the end x
	MOVE.l	#real_max,d7		* get the real maximum bound
	CMP.l		d7,d0				* compare max with new
	BLT.s		no_x_max_bound		* if new < max skip max set

	MOVE.l	d7,d0				* else set new to max
no_x_max_bound
	MOVE.l	d0,rmax(a3)			* save the new maximum real value

	MOVE.w	d1,d0				* copy the first x

	BSR		n_delta			* get the first x * delta
	NEG.l		d0				* make it negative
	ADD.l		rmin(a3),d0			* add the start x
	MOVE.l	#real_min,d7		* get the real minimum bound
	CMP.l		d7,d0				* compare min with new
	BGT.s		no_x_min_bound		* if new > min skip min set

	MOVE.l	d7,d0				* else set new to min
no_x_min_bound
	MOVE.l	d0,rmin(a3)			* save the new minimum real value

* done rmax and rmin, now do imax and imin

							* remember, 0 is the bottom of the screen
	MOVE.w	ysize(a3),d1		* get the y size
	SUBQ.w	#1,d1				* - 1
	MOVE.w	d1,d0				* copy it

	SUB.w		firstY(a3),d1		* get and invert the first y value
	SUB.w		SecondY(a3),d0		* get and invert the second y value

	CMP.w		d1,d0				* compare first with second
	BCC.s		no_y_exg			* if first < second skip the exchange

	EXG		d1,d0				* else exchange first and second y
no_y_exg
	BSR		n_delta			* get the second y * delta
	ADD.l		imax(a3),d0			* add the end y
	MOVE.l	#imag_max,d7		* get the imaginary maximum bound
	CMP.l		d7,d0				* compare max with new
	BLT.s		no_y_max_bound		* if new < max skip max set

	MOVE.l	d7,d0				* else set new to max
no_y_max_bound
	MOVE.l	d0,imax(a3)			* save the new maximum imaginary value

	MOVE.w	d1,d0				* copy the first y

	BSR		n_delta			* get the first y * delta
	NEG.l		d0				* make it negative
	ADD.l		imin(a3),d0			* add the start y
	MOVE.l	#imag_min,d7		* get the imaginary minimum bound
	CMP.l		d7,d0				* compare min with new
	BGT.s		no_y_min_bound		* if new > min skip min set

	MOVE.l	d7,d0				* else set new to min
no_y_min_bound
	MOVE.l	d0,imin(a3)			* save the new minimum imaginary value
	BSR		rubberband			* erase the rubber band
	MOVEQ		#-1,d0			* unset x,y
	MOVE.l	d0,firstX(a3)		* unset the first co-ordinate pair
	MOVE.l	d0,secondX(a3)		* unset the last co-ordinate pair
	BSR		do_delta			* go calculate the new deltas
	BRA		generate			* and generate the fractal

exit_do_unzoom
	RTS


*************************************************************************************
*
* calculate d0.w times the current delta. return the result in d0.l

n_delta
	MOVEM.l	d1-d2,-(sp)			* save the registers
	MOVE.l	delta(a3),d1		* get the delta
	MOVE.w	d1,d2				* copy the low word
	SWAP		d1				* shift the high word to the low word

	MULU		d0,d2				* n x high word d1
	MULU		d1,d0				* low word d1 x n

* done multiply, now add partial products

*			d2 =					aaaa  aaaa	d1_l x n
*			d0 =				----  bbbb		d1_h x n
*			product =				mmmm  mmmm

	SWAP		d0				* align high word
	CLR.w		d0				* clear low word, don't need it
	ADD.l		d2,d0				* add to give product

	MOVEM.l	(sp)+,d1-d2			* restore the registers
	RTS


*************************************************************************************
*
* redraw the fractal

redraw
	MOVE.l	screen_width(a3),d1	* get the screen x,y size
	CMP.l		i_size(a3),d1		* compare it with the rendered image size
	BNE.s		exit_redraw			* if different don't redraw

	LEA		i_buffer(a3),a0		* point to the start of the image buffer
	SUBA.w	xsize(a3),a0		* adjust the image pointer for the loop type
	LEA		palette(pc),a1		* point to the palette

	MOVE.w	ysize(a3),d7		* initial screen y
	SUBQ.w	#1,d7				* adjust for loop type
y_redraw
	MOVE.w	xsize(a3),d6		* initial screen x
	ADDA.w	d6,a0				* increment the image pointer to the next line
	SUBQ.w	#1,d6				* adjust for loop type
x_redraw
	MOVEQ		#0,d0				* clear the longword
	MOVE.b	(a0,d6.w),d0		* get the iteration count from the buffer
	ASL.w		#2,d0				* do iteration count * 4
	MOVE.l	(a1,d0.w),d1		* get the colour
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	d6,d1				* copy the x co-ordinate
	MOVE.w	d7,d2				* copy the y co-ordinate
	MOVEQ		#82,d0			* draw a pixel
	TRAP		#15

	DBF		d6,x_redraw			* loop if more to do

	DBF		d7,y_redraw			* loop if more to do

exit_redraw
	RTS


*************************************************************************************
*
* generate the fractal

generate
	LEA		i_buffer(a3),a0		* point to the start of the image buffer
	SUBA.w	xsize(a3),a0		* adjust the image pointer for the loop type
	LEA		palette(pc),a1		* point to the palette

	MOVEQ		#-1,d7			* set the longword
	MOVE.l	d7,i_size(a3)		* unset the rendered image size

	MOVE.l	imin(a3),imag(a3)		* start at bottom of screen. imag = imin
	ADD.w		ysize(a3),d7		* initial screen y - 1, adjust for loop type
y_loop
	MOVE.l	rmax(a3),real(a3)		* start at left of screen. real = rmax
	MOVE.w	xsize(a3),d6		* initial screen x
	ADDA.w	d6,a0				* increment the image pointer to the next line
	SUBQ.w	#1,d6				* adjust for loop type
x_loop
	BSR		iterate			* calculate this position

	ADDQ.w	#1,d0				* adjust for the iteration loop type
	BEQ.s		colour_set			* if terminal count go set the colour

	NEG.w		d0				* else make negative
	ADD.w		iter(a3),d0			* calculate the actual iteration count
	ADDQ.w	#1,d0				* make the number 1 to n
colour_set
	MOVE.b	d0,(a0,d6.w)		* save the iteration count to the buffer
	ASL.w		#2,d0				* do iteration count * 4
	MOVE.l	(a1,d0.w),d1		* get the colour
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	d6,d1				* copy the x co-ordinate
	MOVE.w	d7,d2				* copy the y co-ordinate
	MOVEQ		#82,d0			* draw a pixel
	TRAP		#15

	MOVEQ		#7,d0				* test for pending key
	TRAP		#15

	TST.b		d1				* test if key waiting
	BEQ.s		no_key			* if no key skip exit test

	MOVEQ		#5,d0				* get the pending key
	TRAP		#15

	CMP.b		#$1B,d1			* compare with [ESC]
	BEQ.s		exit_generate		* if [ESC] exit generate

no_key
	MOVE.l	delta(a3),d0		* get the delta
	SUB.l		d0,real(a3)			* real = real - delta

	DBF		d6,x_loop			* loop if more to do

	ADD.l		d0,imag(a3)			* imag = imag + delta
	DBF		d7,y_loop			* loop if more to do

	MOVE.l	screen_width(a3),i_size(a3)
							* set the rendered image size
exit_generate
	RTS


*************************************************************************************
*
* the palette. EASy68K wants 32 bit colour values as $00BBGGRR a .bmp file wants 32
* bit colour values as $BBGGRR00, this is done by pointing to palette + 1 when saving
* the palette and having an extra $0000 pad word at the end

palette
	dc.l	$00000000,$000000FF,$00000FFF,$00001FFF	*   4
	dc.l	$00002FFF,$00003FFF,$00004FFF,$00005FFF	*   8
	dc.l	$00006FFF,$00007FFF,$00008FFF,$00009FFF	*  12
	dc.l	$0000AFFF,$0000BFFF,$0000CFFF,$0000DFFF	*  16
	dc.l	$0000EFFF,$0000FFFF,$000FFFFF,$001FFFFF	*  20
	dc.l	$002FFFFF,$003FFFFF,$004FFFFF,$005FFFFF	*  24
	dc.l	$006FFFFF,$007FFFFF,$008FFFFF,$009FFFFF	*  28
	dc.l	$00AFFFFF,$00BFFFFF,$00CFFFFF,$00DFFFFF	*  32
	dc.l	$00EFFFFF,$00FFFFFF,$00FFFFEF,$00FFFFDF	*  36
	dc.l	$00FFFFCF,$00FFFFBF,$00FFFFAF,$00FFFF9F	*  40
	dc.l	$00FFFF8F,$00FFFF7F,$00FFFF6F,$00FFFF5F	*  44
	dc.l	$00FFFF4F,$00FFFF3F,$00FFFF2F,$00FFFF1F	*  48
	dc.l	$00FFFF0F,$00FFFF00,$00FFEF00,$00FFDF00	*  52
	dc.l	$00FFCF00,$00FFBF00,$00FFAF00,$00FF9F00	*  56
	dc.l	$00FF8F00,$00FF7F00,$00FF6F00,$00FF5F00	*  60
	dc.l	$00FF4F00,$00FF3F00,$00FF2F00,$00FF1F00	*  64
	dc.l	$00FF0F00,$00FF0000,$00FF000F,$00FF001F	*  68
	dc.l	$00FF002F,$00FF003F,$00FF004F,$00FF005F	*  72
	dc.l	$00FF006F,$00FF007F,$00FF008F,$00FF009F	*  76
	dc.l	$00FF00AF,$00FF00BF,$00FF00CF,$00FF00DF	*  80
	dc.l	$00FF00EF,$00FF00FF,$00EF0FEF,$00DF1FDF	*  84
	dc.l	$00CF2FCF,$00BF3FBF,$00AF4FAF,$009F5F9F	*  88
	dc.l	$008F6F8F,$007F7F7F,$006F8F6F,$005F9F5F	*  92
	dc.l	$004FAF4F,$003FBF3F,$002FCF2F,$001FDF1F	*  96
	dc.l	$000FEF0F,$0000FF00,$0000EF0F,$0000DF1F	* 100
	dc.l	$0000CF2F,$0000BF3F,$0000AF4F,$00009F5F	* 104
	dc.l	$00008F6F,$00007F7F,$00006F8F,$00005F9F	* 108
	dc.l	$00004FAF,$00003FBF,$00002FCF,$00001FDF	* 112
	dc.l	$00000FEF,$000000FF,$00000FFF,$00001FFF	* 116
	dc.l	$00002FFF,$00003FFF,$00004FFF,$00005FFF	* 120
	dc.l	$00006FFF,$00007FFF,$00008FFF,$00009FFF	* 124
	dc.l	$0000AFFF,$0000BFFF,$0000CFFF,$0000DFFF	* 128
	dc.l	$0000EFFF,$0000FFFF,$000FFFFF,$001FFFFF	* 132
	dc.l	$002FFFFF,$003FFFFF,$004FFFFF,$005FFFFF	* 136
	dc.l	$006FFFFF,$007FFFFF,$008FFFFF,$009FFFFF	* 140
	dc.l	$00AFFFFF,$00BFFFFF,$00CFFFFF,$00DFFFFF	* 144
	dc.l	$00EFFFFF,$00FFFFFF,$00FFFFEF,$00FFFFDF	* 148
	dc.l	$00FFFFCF,$00FFFFBF,$00FFFFAF,$00FFFF9F	* 152
	dc.l	$00FFFF8F,$00FFFF7F,$00FFFF6F,$00FFFF5F	* 156
	dc.l	$00FFFF4F,$00FFFF3F,$00FFFF2F,$00FFFF1F	* 160
	dc.l	$00FFFF0F,$00FFFF00,$00FFEF00,$00FFDF00	* 164
	dc.l	$00FFCF00,$00FFBF00,$00FFAF00,$00FF9F00	* 168
	dc.l	$00FF8F00,$00FF7F00,$00FF6F00,$00FF5F00	* 172
	dc.l	$00FF4F00,$00FF3F00,$00FF2F00,$00FF1F00	* 176
	dc.l	$00FF0F00,$00FF0000,$00FF000F,$00FF001F	* 180
	dc.l	$00FF002F,$00FF003F,$00FF004F,$00FF005F	* 184
	dc.l	$00FF006F,$00FF007F,$00FF008F,$00FF009F	* 188
	dc.l	$00FF00AF,$00FF00BF,$00FF00CF,$00FF00DF	* 192
	dc.l	$00FF00EF,$00FF00FF,$00EF0FEF,$00DF1FDF	* 196
	dc.l	$00CF2FCF,$00BF3FBF,$00AF4FAF,$009F5F9F	* 200
	dc.l	$008F6F8F,$007F7F7F,$006F8F6F,$005F9F5F	* 204
	dc.l	$004FAF4F,$003FBF3F,$002FCF2F,$001FDF1F	* 208
	dc.l	$000FEF0F,$0000FF00,$0000EF0F,$0000DF1F	* 212
	dc.l	$0000CF2F,$0000BF3F,$0000AF4F,$00009F5F	* 216
	dc.l	$00008F6F,$00007F7F,$00006F8F,$00005F9F	* 220
	dc.l	$00004FAF,$00003FBF,$00002FCF,$00001FDF	* 224
	dc.l	$00000FEF,$000000FF,$00000FFF,$00001FFF	* 228
	dc.l	$00002FFF,$00003FFF,$00004FFF,$00005FFF	* 232
	dc.l	$00006FFF,$00007FFF,$00008FFF,$00009FFF	* 236
	dc.l	$0000AFFF,$0000BFFF,$0000CFFF,$0000DFFF	* 240
	dc.l	$0000EFFF,$0000FFFF,$000FFFFF,$001FFFFF	* 244
	dc.l	$002FFFFF,$003FFFFF,$004FFFFF,$005FFFFF	* 248
	dc.l	$006FFFFF,$007FFFFF,$008FFFFF,$009FFFFF	* 252
	dc.l	$00AFFFFF,$00BFFFFF,$00CFFFFF,$00DFFFFF	* 256

	dc.w	$0000					* terminating $00 byte for .bmp palette


********************************************************************************
*
* calculate the current r,i iteration given by
*
*					     2   2
*	x = r	   y = i	x   = r + x - y		y   = i + 2xy
*	 0	    0		 n+1	     n   n	 	 n+1		nn
*
* where 2xy is given by
*
*		 2   2   2
*	(x + y) - x - y
*
* as calculating the needed (x + y)^2 is faster than calculating xy
*
* the iteration is performed up to iter(a3) times, but may stop before that if
* SQR(x^2 + y^2) exceeds 2.00 (x^2 + y^2 > 4.00 is actually tested). the value
* returned in d0.l is the remaining number of iterations to do

iterate
	MOVEM.l	d1-d7,-(sp)			* save the registers

	MOVE.l	real(a3),d6			* get the real (x) value
	MOVE.l	imag(a3),d7			* get the imaginary (y) value
	MOVE.w	iter(a3),d0			* get the terminal iteration count
	SUBQ.w	#1,d0				* adjust for loop type

iter_loop
	MOVE.l	d6,d3				* copy the real (x) value
	BSR		square			* do x^2
	MOVE.l	d3,d4				* x^2 in d4

	MOVE.l	d7,d3				* copy the imaginary (y) value
	BSR		square			* do y^2
	MOVE.l	d3,d5				* y^2 in d5

	ADD.l		d4,d3				* calculate the magnitude, x^2 + y^2
	CMP.l		#$10000000,d3		* compare it with the limit, 2.00^2
	BHI.s		exit_iter			* if > 2^2 exit

	MOVE.l	d6,d3				* copy the real (x) value
	ADD.l		d7,d3				* add the imaginary (y) value
	BSR		square			* do (x+y)^2

	MOVE.l	real(a3),d6			* x = real
	ADD.l		d4,d6				* + x^2
	SUB.l		d5,d6				* - y^2

	MOVE.l	imag(a3),d7			* y = imag
	SUB.l		d4,d7				* - x^2
	SUB.l		d5,d7				* - y^2
	ADD.l		d3,d7				* + (x+y)^2

	DBF		d0,iter_loop		* decrement count and loop if more to do

exit_iter
	MOVEM.l	(sp)+,d1-d7			* restore the registers
	RTS


*************************************************************************************
*
* returns the unsigned square of a 6.26 fixed point signed number in d3.l in d3.l

square
	BPL.s		positiv			* if positive skip the negate

	NEG.l		d3				* else negate the number to be squared
positiv
	MOVE.w	d3,d1				* copy low word for b^2
	MULU.w	d1,d1				* do b^2
	CLR.w		d1				* clear the low word
	SWAP		d1				* high word to low word

	MOVE.w	d3,d2				* copy low word again
	SWAP		d3				* high word to low word
	MULU.w	d3,d2				* do a * b
	MULU.w	d3,d3				* do a^2

	ADD.l		d2,d2				* do a * b * 2
	ADD.l		d1,d2				* add the b^2 result
	ADD.l		#$0200,d2			* maybe round up

	AND.w		#$FC00,d2			* clear the discarded bits
	SWAP		d2				* swap words
	ROL.l		#6,d2				* shift it
	ASL.l		#6,d3				* align the partial result
	ADD.l		d2,d3				* calculate the result

	RTS


*************************************************************************************
*
* multiplies a 6.26 fixed point unsigned number in d0.l by a 0.32 fixed point unsigned
* number in d1.l

multiply
	MOVE.w	d0,d4				* copy low word d0
	MOVE.l	d0,d2				* copy long word d0
	SWAP		d2				* high word d0 to low word d0
	MOVE.w	d2,d3				* copy high word d0

	MULU		d1,d0				* low word d1 x low word d0
	MULU		d1,d2				* low word d1 x high word d0
	SWAP		d1				* high word d1 to low word d1
	MULU		d1,d4				* high word d1 x low word d0
	MULU		d1,d3				* high word d1 x high word d0

* done multiply, now add partial products

*			d0 =					aaaa  ----	d1_L x d0_L
*			d2 =				bbbb  aaaa		d1_L x d0_H
*			d4 =				bbbb  aaaa		d1_H x d0_L
*			d3 =			cccc  bbbb			d1_H x d0_H
*			product =		mmmm  mmmm

	ADD.L		#$8000,d0			* round up lowest word
	CLR.w		d0				* clear low word, don't need it
	SWAP		d0				* align high word
	ADD.l		d2,d0				* add d1_L x d0_H (can't be carry)
LAB_MUF1
	ADD.l		d4,d0				* now add intermediate (d1_H x d0_L)
	BCC.s		LAB_MUF2			* branch if no carry

	ADD.l		#$10000,d3			* else correct result
LAB_MUF2
	ADD.l		#$8000,d0			* round up low word
	CLR.w		d0				* clear low word
	SWAP		d0				* align for final add
	ADD.l		d3,d0				* add d1_H x d0_H, result

	RTS


*************************************************************************************
*
* draws the rubber band in EOR mode. if both the first and second x,y pairs are unset
* no band is drawn. if only the second x,y pair is unset the band is drawn from the
* first x,y to the mouse x,y. if both x,y pairs are set then the band is drawn from
* the first x,y to the second x,y

rubberband
	MOVEQ		#14,d1			* set EOR colour is drawn
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	MOVE.l	#$0000FFFF,d1		* yellow, this could be set from a variable
	MOVEQ		#80,d0			* set pen colour
	TRAP		#15

	MOVE.w	firstY(a3),d2		* get y1
	MOVE.w	firstX(a3),d1		* get x1
	BMI.s		no_draw_band		* skip draw if x1 < 0

	MOVE.w	secondY(a3),d4		* get y2
	MOVE.w	secondX(a3),d3		* get x2
	BPL.s		fixed_band			* if set go draw fixed band

	MOVE.w	lastY(a3),d4		* get y2
	MOVE.w	lastX(a3),d3		* get x2
fixed_band
	MOVEQ		#90,d0			* draw unfilled rectangle from x1,y1 to x2,y2
	TRAP		#15

no_draw_band
	MOVEQ		#4,d1				* set normal draw mode
	MOVEQ		#92,d0			* set draw mode
	TRAP		#15

	RTS


*************************************************************************************
*
* initialise the mouse routine. set up the interrupt. for the demo to work it must be
* run with exceptions enabled

mouse_init
	LEA		mouse_interrupt(pc),a0	* get the mouse interrupt routine address
	MOVE.l	a0,$64.w			* save the mouse interrupt vector

	MOVE.w	#$0101,d1			* interrupt for mouse down
	MOVEQ		#60,d0			* set mouse interrupt
	TRAP		#15

	MOVEQ		#0,d0				* clear d0
	MOVE.l	d0,left_button(a3)	* clear both button flags

	RTS


*************************************************************************************
*
* mouse interrupt. read the button states and set the button flag if the corresponding
* button is pressed

mouse_interrupt
	MOVEM.l	d0-d1,-(sp)			* save the registers

	MOVEQ		#2,d1				* mouse down state
	MOVEQ		#61,d0			* read the mouse
	TRAP		#15

	MOVEQ		#-1,d1			* set for flag save
	BTST		#0,d0				* was it the left button
	BEQ.s		not_left_button		* if not skip the flag set

	MOVE.w	d1,left_button(a3)	* set the left button flag
not_left_button
	BTST		#1,d0				* was it the right button
	BEQ.s		not_right_button		* if not skip the flag set

	MOVE.w	d1,right_button(a3)	* set the right button flag
not_right_button
	MOVEM.l	(sp)+,d0-d1			* restore the registers
	RTE


*************************************************************************************
*
* check the mouse buttons. on the left button flag if both the first and second x,y
* pairs are unset then the first pair are set. if only the second x,y pair is unset
* then the second pair is set if both x,y pairs are set then the first pair is set
* and the second set are unset starting a new rubber band. on the right button flag
* both pairs are unset

mouse_buttons
	TST.w		left_button(a3)		* test the left button flag
	BEQ.s		test_right_button		* if not set go test the right button flag

	MOVE.w	#0,left_button(a3)	* clear the left button flag

	TST.w		firstX(a3)			* test if we're drawing a rubberband
	BPL.s		start_set			* if start set go test end

	MOVE.l	lastX(a3),firstX(a3)	* save the mouse x,y as the band first x,y
	BRA.s		test_right_button		* go do right button
	
start_set
	TST.w		secondX(a3)			* test if we've drawn a rubberband
	BPL.s		end_set			* if last set go reset

	MOVE.l	lastX(a3),secondX(a3)	* save the mouse x,y as the band second x,y
	BRA.s		test_right_button		* go do right button

end_set
	BSR		rubberband			* erase the rubber band
	MOVE.l	lastX(a3),firstX(a3)	* save the mouse x,y as the band first x,y
	MOVEQ		#-1,d0			* clear the last x,y
	MOVE.l	d0,secondX(a3)		* clear the last co-ordinate pair
	BSR		rubberband			* draw the rubber band
test_right_button
	TST.w		right_button(a3)		* test the right button flag
	BEQ.s		exit_mouse_buttons	* if not set just exit

	BSR		rubberband			* erase the rubber band
	MOVEQ		#-1,d0			* unset x,y
	MOVE.l	d0,firstX(a3)		* unset the first co-ordinate pair
	MOVE.l	d0,secondX(a3)		* unset the last co-ordinate pair

	MOVE.w	#0,right_button(a3)	* clear the right button flag
exit_mouse_buttons
	RTS


*************************************************************************************
*
* see if the mouse moved, if it did possibly redraw the rubber band. if a fixed
* aspect ratio box is not wanted then the call to aspect and the aspect routine
* should be removed

mouse_move
	MOVEQ 	#0,d1				* read mouse state
	MOVEQ 	#61,d0			* read the mouse
	TRAP		#15

	SWAP		d1				* move mouse y to low word

	TST.w		secondX(a3)			* test if the box is fixed
	BPL.s		mouse_move_exit		* if it is just save the x,y and exit

	BSR		aspect			* go do aspect ratio fix
	CMP.l		lastX(a3),d1		* compare old x,y with new x,y
	BEQ.s		mouse_move_exit		* if the same just exit

	MOVE.l	d1,-(sp)			* save new x,y
	BSR		rubberband			* erase the rubber band
	MOVE.l	(sp)+,d1			* restore new x,y

	MOVE.l	d1,lastX(a3)		* save the new x,y
	BRA		rubberband			* draw the rubber band and return

mouse_move_exit
	MOVE.l	d1,lastX(a3)		* save the new x,y
	RTS


*************************************************************************************
*
* maintain a 4:3 aspect ratio scaled on the relative largest of dx,dy and limited to
* fit into the current window

aspect
	TST.w		firstX(a3)			* is a band started
	BMI		exit_aspect			* if not just exit

	TST.w		secondX(a3)			* is a band done
	BPL		exit_aspect			* if so just exit

* now calculate the absolute dx,dy and flag if they were originally negative

	MOVE.l	d1,d0				* copy x,y
	SWAP		d0				* x to low word

	MOVEQ		#0,d3				* flag both deltas as positive
	SUB.w		firstX(a3),d0		* calculate dx
	BPL.s		dx_not_neg			* branch if not negative

	MOVE.w	#$FF00,d3			* flag negative dx
	NEG.w		d0				* make positive
dx_not_neg
	SUB.w		firstY(a3),d1		* calculate dy
	BPL.s		dy_not_neg			* branch if not negative

	MOVE.b	#$FF,d3			* flag negative dy
	NEG.w		d1				* make positive
dy_not_neg

* make sure dy is a multiple of 3

	EXT.l		d1				* clear the high word
	ADDQ.w	#2,d1				* possibly round up
	DIVU.w	#3,d1				* divide by three
	MULU.w	#3,d1				* multiply by 3

* make sure dx is a multiple of 4

	ADDQ.w	#3,d0				* possibly round up
redo_dx
	ANDI.w	#-4,d0			* mask to a 4 pixel boundary

* now compare 3/4 dx with dy

	MOVE.w	d0,d2				* copy dx
	LSR.w		#1,d2				* / 2
	ADD.w		d0,d2				* make 3/2 dx
	LSR.w		#1,d2				* make 3/4 dx

	CMP.w		d1,d2				* compare dy with 3/4 dx
	BCS.s		set_on_dy			* if dy > 3/4 dx go set the size on dy

* dx was bigger so set dy from dx

	MOVE.w	d2,d1				* set dy as 3/4 dx
	BRA.s		make_last			* go convert the deltas back

* dy was bigger so set dx from dy

set_on_dy
	MOVE.w	#$5555,d0			* 1/3 into d0
	MULU.w	d1,d0				* calculate 1/3 dy
	ADD.l		#$8000,d0			* round up
	SWAP		d0				* result to low word
	ADD.w		d1,d0				* add dy to 1/3 dy

* done the aspect ratio fix now convert back to absolute x,y

make_last
	TST.w		d3				* test dx sign flag
	BPL.s		not_neg_dx			* skip negate if not set

	NEG.w		d0				* make dx negative
not_neg_dx
	TST.b		d3				* test dy sign flag
	BPL.s		not_neg_dy			* skip negate if not set

	NEG.w		d1				* make dy negative
not_neg_dy

* got new signed dx,dy now calculate absolute x,y and check if within the current
* window

	ADD.w		firstX(a3),d0		* calculate x
	BPL.s		x_is_positive		* if positive skip low limit

* else x is below the window minimum so fix it and go recalculate on this x

	MOVE.w	firstX(a3),d0		* get dx maximum
	BRA.s		reset_on_dx			* go recalculate from dx

x_is_positive
	CMP.w		xsize(a3),d0		* compare with max + 1
	BCS.s		x_in_range			* if in range skip high limit

* else x is above the window maximum so fix it and go recalculate on this x

	MOVE.w	xsize(a3),d0		* get maximum size + 1
	SUBQ.w	#1,d0				* - 1
	SUB.w		firstX(a3),d0		* calculate dx maximum
reset_on_dx
	MOVEQ		#1,d1				* make dy smaller
	BRA.s		redo_dx			* go mask to a 4 pixel boundary and
							* recalculate from dx

x_in_range
	ADD.w		firstY(a3),d1		* calculate y
	BPL.s		y_is_positive		* if positive skip low limit

* else y is below the window minimum so fix it and go recalculate on this y

	MOVE.w	firstY(a3),d1		* get dy maximum
	BRA.s		reset_on_dy			* go recalculate from dy

y_is_positive
	CMP.w		ysize(a3),d1		* compare with max + 1
	BCS.s		y_in_range			* if in range skip high limit

* else y is above the window maximum so fix it and go recalculate on this y

	MOVE.w	ysize(a3),d1		* get maximum size + 1
	SUBQ.w	#1,d1				* - 1
	SUB.w		firstY(a3),d1		* calculate dy maximum
reset_on_dy
	DIVU.w	#3,d1				* divide by three
	MULU.w	#3,d1				* multiply by 3
	MOVEQ		#1,d0				* make dx smaller
	BRA.s		set_on_dy			* go recalculate from dy

* x,y are both in range and dx,dy are fixed at a 4:3 ratio so make the x,y longword

y_in_range
	SWAP		d1				* y to the high word
	MOVE.w	d0,d1				* copy the new x
	SWAP		d1				* x to the high word
exit_aspect
	RTS


*************************************************************************************
*
* setup stuff.

Initialise
	LEA		variables(pc),a3		* variables base address
	BSR		mouse_init			* initialise the mouse

	MOVEQ		#-1,d0			* set for no rubberband draw
	MOVE.l	d0,firstX(a3)		* unset the first co-ordinate pair
	MOVE.l	d0,secondX(a3)		* unset the second co-ordinate pair

	MOVE.l	d0,i_size(a3)		* unset the rendered image size

	MOVE.l	#real_max,rmax(a3)	* set the initial real (x) maximum value
	MOVE.l	#real_min,rmin(a3)	* set the initial real (x) minimum value

	MOVE.l	#imag_max,imax(a3)	* set the initial imaginary (y) maximum value
	MOVE.l	#imag_min,imin(a3)	* set the initial imaginary (y) minimum value

	MOVE.w	#iter_min,iter(a3)	* set the initial iteration terminal count

	MOVE.l	#$028001E0,d1		* set default window size to 640 x 480
	BRA		screenset			* go set the screen size and the delta


*************************************************************************************


	INCLUDE	"save.x68"
							* save routines

*************************************************************************************
*
* variables used

variables						* points to variables base

	OFFSET	0				* going to use relative addressing

rmax	ds.l	1					* real (x) upper bound
rmin	ds.l	1					* real (x) lower bound

imax	ds.l	1					* imaginary (y) upper bound
imin	ds.l	1					* imaginary (y) lower bound

real	ds.l	1					* real (x) position
imag	ds.l	1					* imaginary (y) position

delta	ds.l	1					* x,y step delta size

iter	ds.w	1					* iteration terminal count

screen_width
xsize	ds.w	1					* window x size
ysize	ds.w	1					* window y size

left_button
	ds.w	1					* left mouse button flag
right_button
	ds.w	1					* right mouse button flag

firstX
	ds.w	1					* rubberband first x
firstY
	ds.w	1					* rubberband first y
secondX
	ds.w	1					* rubberband second x
secondY
	ds.w	1					* rubberband second y
lastX
	ds.w	1					* last mouse x
lastY
	ds.w	1					* last mouse y

filename_buffer
	ds.b	$100					* save file name buffer
file_id
	ds.l	1					* save file id

i_size
i_xsize
	ds.w	1					* image x size
i_ysize
	ds.w	1					* image y size

i_buffer						* the rest of RAM is the image buffer

*************************************************************************************

	END	start


*************************************************************************************
